from typing import List, Union


from ..core.exception import NotFound
from ..core.http import AsyncHttpHandler
from ..core.models.notifications import Announce

class Announcements:
    def __init__(
        self,
        address: str,
        i: Union[str, None],
        ssl: bool,
        endpoints: List[str],
        handler: AsyncHttpHandler = None,
    ) -> None:
        self.http = handler
        if handler is None:
            self.http = AsyncHttpHandler(address, i, ssl)
        self.endpoints = endpoints

    async def announce(
        self,
        limit: int = 10,
        sinceId: str = None,
        untilId: str = None,
        isActive: bool = True,
    ) -> List[Announce]:
        """Obtains a list of files in the specified folder or root hierarchy under the logged-in user's drive.

        Args:
            limit (int, optional): Maximum number of announcement to retrieve. Defaults to 10.
            sinceId (str, optional): If specified, returns announcement whose id is greater than that value. Defaults to None.
            untilId (str, optional): If specified, returns announcement whose id is less than that value. Defaults to None.
            isActive (Union[str, None], optional): If the id of the parent folder is null (default), files in the root hierarchy are retrieved. Defaults to None.

        Returns:
            List[Announce]: Announcements
        """
        endpoint = "announcements"
        if endpoint not in self.endpoints:
            raise NotFound('endpoint "announcements" is not found.')
        data = {"limit": limit, "isActive": isActive}
        if sinceId:
            data["sinceId"] = sinceId
        if untilId:
            data["untilId"] = untilId
        resp = await self.http.send(endpoint, data=data)
        anmts = []
        for res in resp:
            anmts.append(Announce(**res))
        return anmts

    async def show(self, announcementId: str) -> Announce:
        """Show announcements

        Args:
            announcementId (str): announcement id.

        Returns:
            Announce: announce body
        """
        endpoint = "announcements/show"
        if endpoint not in self.endpoints:
            raise NotFound('endpoint "announcements/show" is not found.')
        return Announce(**await self.http.send(endpoint, data={"announcementId": announcementId}))



from __future__ import annotations

from typing import Union

from pydantic import dataclasses

@dataclasses.dataclass()
class Channel:
    id: str
    name: str
    color: str
    isSensitive: bool
    allowRenoteToExternal: bool
    userId: Union[str, None]



from typing import List, Union

from ..core.http import AsyncHttpHandler

class reactions:
    def __init__(
        self, address: str, i: Union[str, None], ssl: bool, endpoints: List[str], handler: AsyncHttpHandler=None
    ) -> None:
        self.http = handler
        if handler is None:
            self.http = AsyncHttpHandler(address, i, ssl)
        self.endpoints = endpoints

    async def create(self, reaction: str, noteId: str) -> None:
        """create reaction.

        Args:
            address (string): instance address
            i (string): user token
            noteId (string): noteId
            reaction (string): Specify reaction. Reactions are Unicode emojis or custom emojis. For custom emoji, enclose the emoji name with a colon.

        Returns:
            dict: Misskey API response
        """
        return await self.http.send(
            "notes/reactions/create", {"noteId": noteId, "reaction": reaction}
        )


    async def delete(self, noteId) -> None:
        """delete reaction.

        Args:
            address (string): instance address
            i (string): user token
            noteId (string): noteId

        Returns:
            dict: Misskey API response
        """
        return await self.http.send("notes/reactions/delete", {"noteId": noteId})


from typing import Union, List

from pydantic import dataclasses
from mitypes.user import AvatarDecorations

from .note import Note
from .page import Page
from .role import RoleLite

@dataclasses.dataclass
class Emojis:
    ""


@dataclasses.dataclass
class Instance:
    name: Union[str, None]
    softwareName: Union[str, None]
    softwareVersion: Union[str, None]
    iconUrl: Union[str, None]
    faviconUrl: Union[str, None]
    themeColor: Union[str, None]


@dataclasses.dataclass
class BadgeRoles:
    name: str
    iconUrl: Union[str, None]
    displayOrder: int
    behavior: str


@dataclasses.dataclass
class field:
    name: str
    value: str

@dataclasses.dataclass
class users:
    id: str
    name: Union[str, None]
    username: str
    host: Union[str, None]
    avatarUrl: Union[str, None]
    avatarBlurhash: Union[str, None]
    avatarDecorations: AvatarDecorations
    isBot: bool
    isCat: bool
    instance: Instance
    emojis: Emojis
    onlineStatus: str
    badgeRoles: BadgeRoles
    url: Union[str, None]
    uri: Union[str, None]
    movedTo: Union[str, None]
    alsoKnownAs: List[Union[str, None]]
    createdAt: str
    updatedAt: Union[str, None]
    lastFetchedAt: Union[str, None]
    bannerUrl: Union[str, None]
    bannerBlurhash: Union[str, None]
    isLocked: bool
    isSilenced: bool
    isLimited: bool
    isSuspended: bool
    description: Union[str, None]
    location: Union[str, None]
    birthday: Union[str, None]
    lang: Union[str, None]
    fields: field
    verifiedLinks: List[str]
    followersCount: int
    followingCount: int
    notesCount: int
    pinnedNoteIds: List[str]
    pinnedNotes: List[Note]
    pinnedPageId: str
    pinnedPage: Page
    publicReactions: bool
    followingVisibility: str
    followersVisibility: str
    roles: RoleLite
    memo: Union[str, None]
    moderationNote: str
    isFollowing: bool
    isFollowed: bool
    hasPendingFollowRequestFromYou: bool
    hasPendingFollowRequestToYou: bool
    isBlocking: bool
    isBlocked: bool
    isMuted: bool
    isRenoteMuted: bool
    notify: str
    withReplies: bool
    twoFactorEnabled: bool = False
    usePasswordLessLogin: bool = False
    securityKeys: bool = False


import sys
from datetime import datetime
from typing import Any, List, Union

from mitypes.drive import DriveFile
from mitypes.user import AvatarDecorations, UserLite
from pydantic.dataclasses import dataclass, Field

from .action import APIAction
from .channel import Channel
# from .federation import BadgeRoles, Emojis, Instance
from .internal import mspy


@dataclass()
class Context:
    id: str
    createdAt: str
    userId: str
    user: UserLite
    localOnly: bool
    renoteCount: int
    repliesCount: int
    clippedCount: int
    reactionCount: int
    reactionEmojis: dict
    reactionAndUserPairCache: List[str]
    replyId: Union[str, None] = None
    renoteId: Union[str, None] = None
    visibility: str = None
    reactions: Union[dict, None] = None
    uri: Union[str, None] = None
    url: Union[str, None] = None
    isHidden: Union[bool, None] = None
    deletedAt: Union[str, None] = None
    text: Union[str, None] = None
    cw: Union[str, None] = None
    reply: Union[dict, None] = None
    renote: Union[dict, None] = None
    mentions: List[str] = Field(default_factory=list)
    visibleUserIds: List[str] = Field(default_factory=list)
    fileIds: List[str] = Field(default_factory=list)
    files: List[DriveFile] = Field(default_factory=list)
    tags: List[str] = Field(default_factory=list)
    poll: Union[dict, None] = None
    channelId: Union[str, None] = None
    channel: Union[dict, None] = None
    reactionAcceptance: Union[str, None] = None
    reactionAndUserPairCache: List[str] = Field(default_factory=list)
    misspy: Union[mspy, None] = None
    api: Union[APIAction, None] = None


@dataclass()
class Note:
    id: str
    createdAt: str
    userId: str
    user: UserLite
    visibility: str
    localOnly: bool
    uri: str
    url: str
    reactions: dict
    renoteCount: int
    repliesCount: int
    reactionCount: int
    clippedCount: int
    reactionEmojis: dict
    reactionAndUserPairCache: List[str]
    replyId: Union[str, None] = None
    renoteId: Union[str, None] = None
    myReaction: Union[dict, None] = None

    isHidden: Union[bool, None] = None
    deletedAt: Union[str, None] = None
    text: Union[str, None] = None
    cw: Union[str, None] = None
    reply: Union[dict, None] = None
    renote: Union[dict, None] = None
    mentions: List[str] = Field(default_factory=list)
    visibleUserIds: List[str] = Field(default_factory=list)
    fileIds: List[str] = Field(default_factory=list)
    files: List[DriveFile] = Field(default_factory=list)
    tags: List[str] = Field(default_factory=list)
    poll: Union[dict, None] = None
    channelId: Union[str, None] = None
    channel: Union[Channel, None] = None
    reactionAcceptance: Union[str, None] = None
    reactionAndUserPairCache: List[str] = Field(default_factory=list)
    misspy: Union[mspy, None] = None

    def __post_init__(self):
        verinf = sys.version_info
        if not verinf.major + verinf.minor >= 311:
            self.createdAt = datetime.fromisoformat(
                self.createdAt.replace("Z", "+00:00")
            )
            if self.deletedAt:
                self.deletedAt = datetime.fromisoformat(
                    self.deletedAt.replace("Z", "+00:00")
                )
        else:
            self.createdAt = datetime.fromisoformat(self.createdAt)
            if self.deletedAt:
                self.deletedAt = datetime.fromisoformat(self.deletedAt)


from typing import List, Union

from mitypes import DriveFile, UserLite
from pydantic import dataclasses


@dataclasses.dataclass()
class posts:
    id: str
    createdAt: str
    updatedAt: str
    userId: str
    user: UserLite
    title: str
    description: Union[str, None]
    fileIds: List[str]
    files: DriveFile
    tags: List[str]
    isSensitive: bool
    likedCount: int
    isLikes: bool



from typing import List, Union

from mitypes import User, UserLite
from mitypes.drive import DriveFile

from ..core.http import AsyncHttpHandler, HttpHandler
from ..core.models.antennas import created_antnna, notes, show_antnna
from ..core.models.blocking import blocking_list
from ..core.models.clip import clips
from ..core.models.favorites import favorite
from ..core.models.following import RequestList
from ..core.models.likes import GalleryPost, Like
from ..core.models.mute import mute as mp_mute
from ..core.models.note import Note  # , Pin
from ..core.models.notifications import notifications as notify
from ..core.models.page import Page, likes
from ..core.models.users import (
    Follow,
    Frequently_replied,
    MeDetailed,
    Relation,
    UserDetailedNotMe,
    UserStat,
    followers,
    reactions,
)
from ..utils.internaltool import nonecheck


class i:
    def __init__(
        self,
        address: str,
        i: Union[str, None],
        ssl: bool,
        endpoints: List[str],
        handler: AsyncHttpHandler = None,
    ) -> None:
        self.i = i
        self.address = address
        self.ssl = ssl
        self.__http = handler

    async def get(self):
        r = await self.__http.send("i", {})
        return User(**r)

    async def favorites(self, limit=10, sinceId=None, untilId=None):
        base = {"limit": limit}
        if nonecheck(sinceId):
            base["sinceId"] = sinceId
        if nonecheck(untilId):
            base["untilId"] = untilId
        return [favorite(**fav) for fav in await self.__http.send("i/favorites", base)]

    async def gallery_likes(self, limit=10, sinceId=None, untilId=None):
        base = {"limit": limit}
        if nonecheck(sinceId):
            base["sinceId"] = sinceId
        if nonecheck(untilId):
            base["untilId"] = untilId
        resp = await self.__http.send("i/gallery/likes", base)
        likes = []
        for like in resp:
            like["post"]["files"] = [DriveFile(**fav) for fav in resp["files"]]
            likes.append(Like(**like))
        return likes

    async def gallery_posts(self, limit=10, sinceId=None, untilId=None):
        base = {"limit": limit}
        if nonecheck(sinceId):
            base["sinceId"] = sinceId
        if nonecheck(untilId):
            base["untilId"] = untilId
        resp = await self.__http.send("i/gallery/posts", base)
        posts = []
        for post in resp:
            post["files"] = [DriveFile(**fav) for fav in resp["files"]]
            posts.append(GalleryPost(**post))
        return posts

    async def notifications(
        self,
        limit=10,
        sinceId=None,
        untilId=None,
        following=False,
        unreadOnly=False,
        markAsRead=True,
        includeTypes=None,
        excludeTypes=None,
    ):
        base = {
            "limit": limit,
            "following": following,
            "unreadOnly": unreadOnly,
            "markAsRead": markAsRead,
        }
        if nonecheck(sinceId):
            base["sinceId"] = sinceId
        if nonecheck(untilId):
            base["untilId"] = untilId
        if nonecheck(includeTypes):
            base["includeTypes"] = includeTypes
        if nonecheck(excludeTypes):
            base["excludeTypes"] = excludeTypes
        return [
            notify(**notif) for notif in await self.__http.send("i/notifications", base)
        ]

    async def page_likes(self, limit=10, sinceId=None, untilId=None):
        base = {"limit": limit}
        if nonecheck(sinceId):
            base["sinceId"] = sinceId
        if nonecheck(untilId):
            base["untilId"] = untilId
        return [
            Page(**page_like)
            for page_like in await self.__http.send("i/page-likes", base)
        ]

    async def pages(self, limit=10, sinceId=None, untilId=None):
        base = {"limit": limit}
        if nonecheck(sinceId):
            base["sinceId"] = sinceId
        if nonecheck(untilId):
            base["untilId"] = untilId
        resp = await self.__http.send("i/page-likes", base)
        lks = []
        for page_like in resp:
            lks.append(likes(**page_like))
        return lks

    """
    async def pin(self, noteId):
        return Pin(**await self.__http.send("i/pin", {"noteId": noteId}))

    async def unpin(self, noteId):
        return Pin(**await self.__http.send("i/unpin", {"noteId": noteId}))

    async def update(self, params):
        return Pin(**await self.__http.send("i/update", params))
    """

    async def read_all_unread_notes(self):
        return await self.__http.send("i/read-all-unread-notes", {})

    async def read_announcement(self, announcementId):
        return await self.__http.send(
            "i/read-all-unread-notes",
            {"announcementId": announcementId},
        )


class blocking:
    def __init__(
        self,
        address: str,
        i: Union[str, None],
        ssl: bool,
        endpoints: List[str],
        handler: AsyncHttpHandler = None,
    ) -> None:
        self.__http = handler

    async def create(self, userId: str):
        params = {"userId": userId}
        r = await self.__http.send("blocking/create", params)
        return UserDetailedNotMe(**r)

    async def delete(self, userId: str):
        params = {"userId": userId}
        r = await self.__http.send("blocking/delete", params)
        return r

    async def list(self, limit: int = 30, sinceId: str = None, untilId: str = None):
        params = {
            "limit": limit,
        }
        if sinceId is not None:
            params["sinceId"] = sinceId
        if untilId is not None:
            params["untilId"] = untilId
        r = await self.__http.send("blocking/list", params)
        return [blocking_list(**page_like) for page_like in r]


class antennas:
    def __init__(
        self,
        address: str,
        i: Union[str, None],
        ssl: bool,
        endpoints: List[str],
        handler: AsyncHttpHandler = None,
    ) -> None:
        self.i = i
        self.address = address
        self.__http = handler
        self.endpoints = endpoints

    async def create(
        self,
        name,
        src,
        keywords,
        excludeKeywords,
        users,
        caseSensitive,
        withReplies,
        withFile,
        notify,
        userListId=None,
    ):
        base = {
            "name": name,
            "src": src,
            "keywords": keywords,
            "excludeKeywords": excludeKeywords,
            "users": users,
            "caseSensitive": caseSensitive,
            "withReplies": withReplies,
            "withFile": withFile,
            "notify": notify,
        }
        if nonecheck(userListId):
            base["userListId"] = userListId
        return created_antnna(**await self.__http.send("antennas/create", base))

    async def update(
        self,
        name,
        src,
        keywords,
        excludeKeywords,
        users,
        caseSensitive,
        withReplies,
        withFile,
        notify,
        userListId=None,
    ):
        base = {
            "name": name,
            "src": src,
            "keywords": keywords,
            "excludeKeywords": excludeKeywords,
            "users": users,
            "caseSensitive": caseSensitive,
            "withReplies": withReplies,
            "withFile": withFile,
            "notify": notify,
        }
        if nonecheck(userListId):
            base["userListId"] = userListId
        return created_antnna(**self.__http.send("antennas/update", base))

    async def delete(self, antennaId):
        return self.__http.send("antennas/delete", {"antennaId": antennaId})

    async def show(self, antennaId):
        return show_antnna(
            **self.__http.send("antennas/show", {"antennaId": antennaId})
        )

    async def list(self):
        return [show_antnna(**r) for r in await self.__http.send("antennas/list", {})]

    async def notes(
        self,
        antennaId,
        limit=10,
        sinceId=None,
        untilId=None,
        sinceDate=None,
        untilDate=None,
    ):
        base = {"antennaId": antennaId, "limit": limit}
        if nonecheck(sinceId):
            base["sinceId"] = sinceId
        if nonecheck(untilId):
            base["untilId"] = untilId
        if nonecheck(sinceDate):
            base["sinceDate"] = sinceDate
        if nonecheck(untilDate):
            base["untilDate"] = untilDate
        return [notes(**r) for r in await self.__http.send("antennas/notes", base)]


class mute:
    def __init__(
        self,
        address: str,
        i: Union[str, None],
        ssl: bool,
        endpoints: List[str],
        handler: AsyncHttpHandler = None,
    ) -> None:
        self.__http = handler

    async def create(self, userId, expiresAt=None):
        return await self.__http.send(
            "mute/create",
            {"userId": userId, "expiresAt": expiresAt},
        )

    async def list(self, limit=30, sinceId=None, untilId=None):
        base = {"limit": limit}
        if nonecheck(sinceId):
            base["sinceId"] = sinceId
        if nonecheck(untilId):
            base["untilId"] = untilId
        return [mp_mute(**r) for r in await self.__http.send("mute/list", base)]

    async def delete(self, userId):
        return await self.__http.send("mute/delete", {"userId": userId})


class users:
    def __init__(
        self,
        address: str,
        i: Union[str, None],
        ssl: bool,
        endpoints: List[str],
        handler: AsyncHttpHandler = None,
    ) -> None:
        self.i = i
        self.address = address
        self.__http = handler
        self.__http_sync = HttpHandler(address, i, ssl)

    async def get(
        self, limit=10, offset=0, sort=None, state="all", origin="local", hostname=None
    ):
        base = {
            "limit": limit,
            "offset": offset,
            "state": state,
            "origin": origin,
            "hostname": hostname,
        }
        if nonecheck(sort):
            base["sort"] = sort
        resp = await self.__http.send("users", base)
        rl = []
        for r in resp:
            if r.get("loggedInDays") is None:
                rl.append(MeDetailed(**r))
            else:
                rl.append(UserDetailedNotMe(**r))
        return rl

    async def clips(self, userId, limit=10, sinceId=None, untilId=None):
        base = {"userId": userId, "limit": limit}
        if nonecheck(sinceId):
            base["sinceId"] = sinceId
        if nonecheck(untilId):
            base["untilId"] = untilId
        return [clips(**r) for r in await self.__http.send("users/clips", base)]

    async def followers(self, limit=10, sinceId=None, untilId=None):
        base = {"limit": limit}
        if nonecheck(sinceId):
            base["sinceId"] = sinceId
        if nonecheck(untilId):
            base["untilId"] = untilId
        return [followers(**r) for r in await self.__http.send("users/followers", base)]

    async def following(self, limit=10, sinceId=None, untilId=None):
        base = {"limit": limit}
        if nonecheck(sinceId):
            base["sinceId"] = sinceId
        if nonecheck(untilId):
            base["untilId"] = untilId
        return [followers(**r) for r in await self.__http.send("users/following", base)]

    async def gallery_posts(self, userId, limit=10, sinceId=None, untilId=None):
        base = {"userId": userId, "limit": limit}
        if nonecheck(sinceId):
            base["sinceId"] = sinceId
        if nonecheck(untilId):
            base["untilId"] = untilId
        return [
            GalleryPost(**r)
            for r in await self.__http.send("users/gallery/posts", base)
        ]

    async def pages(self, userId, limit=10, sinceId=None, untilId=None):
        base = {"userId": userId, "limit": limit}
        if nonecheck(sinceId):
            base["sinceId"] = sinceId
        if nonecheck(untilId):
            base["untilId"] = untilId
        return [Page(**r) for r in await self.__http.send("users/pages", base)]

    async def reactions(
        self,
        userId,
        limit=10,
        sinceId=None,
        untilId=None,
        sinceDate=None,
        untilDate=None,
    ):
        base = {"userId": userId, "limit": limit}
        if nonecheck(sinceId):
            base["sinceId"] = sinceId
        if nonecheck(untilId):
            base["untilId"] = untilId
        if nonecheck(sinceDate):
            base["sinceDate"] = sinceDate
        if nonecheck(untilDate):
            base["untilDate"] = untilDate
        return [reactions(**r) for r in await self.__http.send("users/reactions", base)]

    async def recommendation(
        self,
        limit=10,
        offset=0,
    ):
        base = {"limit": limit, "offset": offset}
        r = await self.__http.send("users/recommendation", base)
        ret = r
        if not isinstance(r, bool):
            ret = []
            for res in r:
                if res.get("loggedInDays"):
                    ret.append(MeDetailed(res))
                else:
                    ret.append(UserDetailedNotMe(res))
        return ret

    async def relation(self, userId):
        r = await self.__http.send("users/relation", {"userId": userId})
        if isinstance(r, dict):
            return Relation(**r)
        elif isinstance(r, list):
            return [Relation(**res) for res in r]

    async def report_abuse(self, userId, comment):
        return await self.__http.send(
            "users/report-abuse",
            {"userId": userId, "comment": comment},
        )

    async def search_by_username_and_host(
        self, username=None, host=None, limit=10, detail=True
    ) -> Union[UserLite, MeDetailed, UserDetailedNotMe]:
        r = await self.__http.send(
            "users/search-by-username-and-host",
            {"username": username, "host": host, "limit": limit, "detail": detail},
        )
        if r.get("alsoKnownAs") is None:
            return UserLite(**r)
        elif r.get("alsoKnownAs"):
            if r.get("loggedInDays"):
                return MeDetailed(**r)
            elif r.get("loggedInDays") is None:
                return UserDetailedNotMe(**r)

    async def search(
        self, query, offset=0, limit=10, origin="combined", detail=True
    ) -> Union[UserLite, MeDetailed, UserDetailedNotMe]:
        r = await self.__http.send(
            "users/search",
            {
                "query": query,
                "offset": offset,
                "limit": limit,
                "origin": origin,
                "detail": detail,
            },
        )
        if r.get("alsoKnownAs") is None:
            return UserLite(**r)
        else:
            if r.get("loggedInDays") is None:
                return UserDetailedNotMe(**r)
            return MeDetailed(**r)

    def show(
        self, username, host=None
    ) -> Union[
        UserLite,
        MeDetailed,
        UserDetailedNotMe,
        List[UserLite],
        List[MeDetailed],
        List[UserDetailedNotMe],
    ]:
        r = self.__http.send("users/show", {"username": username, "host": None})
        if isinstance(r, dict):
            if r.get("alsoKnownAs") is None:
                return UserLite(**r)
            else:
                if r.get("loggedInDays") is None:
                    return UserDetailedNotMe(**r)
                return MeDetailed(**r)
        elif isinstance(r, list):
            rl = []
            for res in r:
                if res.get("alsoKnownAs") is None:
                    rl.append(UserLite(**r))
                else:
                    if res.get("loggedInDays") is None:
                        rl.append(UserDetailedNotMe(**r))
                    rl.append(MeDetailed(**r))
            return rl

    async def stats(self, userId):
        r = await self.__http.send("users/stats", {"userId": userId})
        usage = r["driveUsage"]
        r["driveUsage"] = {
            "byte": usage,
            "kilobyte": usage * 1024,
            "megabyte": (usage * 1024) * 1024,
            "gigabyte": ((usage * 1024) * 1024) * 1024,
            "terabyte": (((usage * 1024) * 1024) * 1024) * 1024,
        }
        return UserStat(**r)

    async def get_frequently_replied_users(
        self,
        userId,
        limit=10,
    ):
        base = {"userId": userId, "limit": limit}
        return [
            Frequently_replied(**r)
            for r in await self.__http.send("users/get-frequently-replied-users", base)
        ]

    async def notes(
        self,
        userId,
        includeReplies=True,
        limit=10,
        sinceId=None,
        untilId=None,
        sinceDate=None,
        untilDate=None,
        includeMyRenotes=True,
        withFiles=False,
        fileType=None,
        excludeNsfw=False,
    ):
        base = {
            "userId": userId,
            "includeReplies": includeReplies,
            "limit": limit,
            "includeMyRenotes": includeMyRenotes,
            "withFiles": withFiles,
            "excludeNsfw": excludeNsfw,
        }
        if nonecheck(sinceDate):
            base["sinceDate"] = sinceDate
        if nonecheck(untilDate):
            base["untilDate"] = untilDate
        if nonecheck(withFiles):
            base["withFiles"] = withFiles
        if nonecheck(fileType):
            base["fileType"] = fileType
        if nonecheck(sinceId):
            base["sinceId"] = sinceId
        if nonecheck(untilId):
            base["untilId"] = untilId
        return [Note(**r) for r in await self.__http.send("users/notes", base)]


class following:
    def __init__(
        self,
        address: str,
        i: Union[str, None],
        ssl: bool,
        endpoints: List[str],
        handler: AsyncHttpHandler = None,
    ) -> None:
        self.__http = handler

    async def create(self, userId) -> Follow:
        return Follow(**await self.__http.send("following/create", {"userId": userId}))

    async def delete(self, userId) -> Follow:
        return Follow(**await self.__http.send("following/delete", {"userId": userId}))

    async def invalidate(self, userId) -> Follow:
        return Follow(
            await self.__http.send("following/invalidate", {"userId": userId})
        )

    async def requests_accept(self, userId) -> bool:
        return await self.__http.send("following/requests/accept", {"userId": userId})

    async def requests_cancel(self, userId) -> Follow:
        return Follow(
            await self.__http.send("following/requests/cancel", {"userId": userId})
        )

    async def requests_list(self) -> List[RequestList]:
        return [
            RequestList(**r)
            for r in await self.__http.send("following/requests/list", {})
        ]

    async def requests_reject(self, userId) -> bool:
        return await self.__http.send("following/requests/reject", {"userId": userId})



from __future__ import annotations

from typing import Any, List, Union

from mitypes import UserLite
from mitypes.drive import DriveFile
from mitypes.poll import Poll
from pydantic import dataclasses

from .channel import Channel
from .federation import Emojis


@dataclasses.dataclass()
class notes:
    id: str
    createdAt: str
    deletedAt: Union[str, None]
    text: Union[str, None]
    cw: Union[str, None]
    userId: str
    user: UserLite
    replyId: Union[str, None]
    renoteId: Union[str, None]
    reply: notes
    renote: notes
    isHidden: bool
    visibility: str
    mentions: List[str]
    visibleUserIds: List[str]
    fileIds: List[str]
    files: List[DriveFile]
    tags: List[str]
    poll: Poll
    emojis: Emojis
    channelId: Union[str, None]
    channel: Channel
    localOnly: bool
    reactionAcceptance: Union[str, None]
    reactionEmojis: Emojis
    reactions: Any
    reactionCount: int
    renoteCount: int
    renoteCount: int
    repliesCount: int
    uri: str
    url: str
    reactionAndUserPairCache: List[str]
    clippedCount: int
    myReaction: Union[str, None]


@dataclasses.dataclass()
class created_antnna:
    id: str
    createdAt: str
    name: str
    keywords: List[str]
    excludeKeywords: List[str]
    src: str
    userListId: Union[str, None]
    users: List[str]
    notify: bool
    withFile: bool
    isActive: bool
    caseSensitive: bool = False
    localOnly: bool = False
    excludeBots: bool = False
    withReplies: bool = False
    hasUnreadNote: bool = False


@dataclasses.dataclass()
class show_antnna:
    id: str
    createdAt: str
    name: str
    keywords: List[str]
    excludeKeywords: List[str]
    src: str
    userListId: Union[str, None]
    users: List[str]
    notify: bool
    withFile: bool
    isActive: bool
    caseSensitive: bool = False
    localOnly: bool = False
    excludeBots: bool = False
    withReplies: bool = False
    hasUnreadNote: bool = False



import urllib.parse
import uuid
from typing import List, Union

import aiohttp
from mitypes import User

from ...core.exception import MiAuthFailed, NotFound
from ...core.models.internal import miauth_session


class session:
    def __init__(self) -> None:
        pass

    async def generate(
        self,
        host: str,
        name: Union[str, None] = None,
        icon: Union[str, None] = None,
        callback: Union[str, None] = None,
        permission: Union[List[str], None] = None,
    ):
        sessionId = uuid.uuid4()
        url = urllib.parse.urlparse("https://{}".format(host))
        params = urllib.parse.parse_qs(url.query)
        if name:
            params["name"] = name
        if icon:
            params["icon"] = icon
        if callback:
            params["callback"] = callback
        if permission:
            params["permission"] = ",".join(permission)
        generated_session = miauth_session(
            sessionId,
            host,
            urllib.parse.urlunparse(
                url._replace(query=urllib.parse.urlencode(params, doseq=True))
            ),
            name,
            icon,
            callback,
            permission,
        )
        return generated_session

    async def check(self, authSession: miauth_session):
        async with aiohttp.ClientSession() as session:
            async with session.post(
                "https://{}/api/miauth/{}/check".format(
                    authSession.host, authSession.sessionId
                )
            ) as resp:
                try:
                    resp = await resp.json()
                except aiohttp.ContentTypeError:
                    raise NotFound(
                        'API endpoint "miauth/{}/check" does not exist. The version of Misskey you are using may be older than 12.27.0.'.format(
                            authSession.sessionId
                        )
                    )
                resp_i: dict = resp["user"]
                resp_i["token"] = resp["token"]
                if resp.get("token") is not None:
                    return User(**resp_i)
                else:
                    raise MiAuthFailed(resp)



from pydantic import dataclasses

from .users import UserDetailedNotMe


@dataclasses.dataclass()
class blocking_list:
    id: str
    createdAt: str
    blockeeId: str
    blockee: UserDetailedNotMe



import logging
from typing import Union

import aiohttp
import orjson
import requests

from .exception import ClientException, MisskeyAPIError


class AsyncHttpHandler:
    def __init__(
        self,
        address: str,
        i: Union[str, None] = None,
        ssl=True,
        logger: Union[logging.Logger, None] = None,
        logger_level: int = logging.INFO,
    ) -> None:
        self.logger = logger
        logging.basicConfig(level=logger_level)
        self.i = i
        self.address = address
        self.session = None
        self.header = {"Content-Type": "application/json"}
        self.urlfmt = "://"
        if ssl:
            self.urlfmt = "s://"

    async def send(
        self, endpoint: str, data: dict, header: Union[str, None] = None, file=None
    ) -> dict:
        """addressで指定されたサーバーにPOSTリクエストを送信します。

        Args:
            endpoint (str): エンドポイント (例: notes/create)
            data (dict): 送信するデータ。トークン(i)は自動的に挿入される為不要です。
            header (Union[str, None], optional): ヘッダー。ファイルが添付されている場合は利用されません。 Defaults to None.
            file (_type_, optional): ファイル。 Defaults to None.

        Raises:
            MisskeyAPIError: 何らかの理由でMisskeyAPIからエラーが返された場合発生します。

        Returns:
            dict: リクエスト結果
        """
        if self.session is None:
            self.session = aiohttp.ClientSession()
        if header is None:
            header = self.header
        if self.i is not None:
            data["i"] = self.i
        if file is not None:
            resp = await self.session.post(
                "http" + self.urlfmt + self.address + "/api/" + endpoint,
                data=orjson.dumps(data).decode("utf-8"),
            )
        else:
            resp = await self.session.post(
                "http" + self.urlfmt + self.address + "/api/" + endpoint,
                data=orjson.dumps(data).decode("utf-8"),
                headers=header,
            )
        if not resp.status == 200:
            try:
                errorm = await resp.json()
                if errorm["error"]["kind"] == "client":
                    raise ClientException(
                        errorm["error"]["message"]
                        + "\nERROR_CODE: "
                        + errorm["error"]["code"]
                        + "\nINFO: "
                        + errorm["error"]["info"]
                    )
                raise MisskeyAPIError(
                    errorm["error"]["message"]
                    + "\nERROR_CODE: "
                    + errorm["error"]["code"]
                    + "\nINFO: "
                    + errorm["error"]["info"]
                )
            except:  # noqa: E722
                errorm = orjson.loads(await resp.text())
                if errorm["error"]["kind"] == "client":
                    raise ClientException(
                        errorm["error"]["message"]
                        + "\nERROR_CODE: "
                        + errorm["error"]["code"]
                        + "\nINFO: "
                        + orjson.dumps(errorm["error"]["info"]).decode("utf-8")
                    )
                raise MisskeyAPIError(
                    errorm["error"]["message"]
                    + "\nERROR_CODE: "
                    + errorm["error"]["code"]
                    + "\nINFO: "
                    + errorm["error"]["info"]
                )
        try:
            return await resp.json()
        except:  # noqa: E722
            return True


class HttpHandler:
    def __init__(self, address: str, i: Union[str, None] = None, ssl=True) -> None:
        self.i = i
        self.address = address
        self.session = requests.Session()
        self.header = {"Content-Type": "application/json"}
        self.urlfmt = "://"
        if ssl:
            self.urlfmt = "s://"

    def send(
        self, endpoint: str, data: dict, header: Union[str, None] = None, file=None
    ) -> dict:
        if header is None:
            header = self.header
        if self.i is not None:
            data["i"] = self.i

        if file is not None:
            resp = self.session.post(
                "http" + self.urlfmt + self.address + "/api/" + endpoint,
                data=orjson.dumps(data).decode("utf-8"),
            )
        else:
            resp = self.session.post(
                "http" + self.urlfmt + self.address + "/api/" + endpoint,
                data=orjson.dumps(data).decode("utf-8"),
                headers=header,
            )
        if not resp.status_code == 200:
            try:
                raise MisskeyAPIError(resp.json())
            except requests.exceptions.JSONDecodeError:
                raise MisskeyAPIError(resp.text)
        return resp.json()



from typing import Any, List, Union

from mitypes import UserLite
from mitypes.user import AvatarDecorations
from pydantic import dataclasses
from pydantic.dataclasses import Field

from .federation import BadgeRoles, Emojis, Instance, field
from .note import Note
from .other import Achievements, Announcement, RolePolicies, SecurityKeysList
from .page import Page
from .role import RoleLite

@dataclasses.dataclass()
class DriveUsage:
    byte: int
    kilobyte: int
    megabyte: int
    gigabyte: int
    terabyte: int


@dataclasses.dataclass()
class UserStat:
    notesCount: int
    repliesCount: int
    renotesCount: int
    repliedCount: int
    renotedCount: int
    pollVotesCount: int
    pollVotedCount: int
    localFollowingCount: int
    remoteFollowingCount: int
    localFollowersCount: int
    remoteFollowersCount: int
    followingCount: int
    followersCount: int
    sentReactionsCount: int
    recivedReactionsCount: int
    noteFavoritesCount: int
    pageLikesCount: int
    pageLikedCount: int
    driveFilesCount: int
    driveUsage: DriveUsage


@dataclasses.dataclass()
class Relation:
    id: str
    isFollowing: bool
    hasPnedingFollowRequestFromYou: bool
    hasPendingFollowRequestToYou: bool
    isFollowed: bool
    isBlocking: bool
    isBlocked: bool
    isMuted: bool
    isRenoteMuted: bool


@dataclasses.dataclass()
class MeDetailed:
    id: str
    name: Union[str, None]
    username: str
    host: Union[str, None]
    avatarUrl: Union[str, None]
    avatarBlurhash: Union[str, None]
    avatarDecorations: AvatarDecorations
    isBot: bool
    isCat: bool
    instance: Instance
    emojis: Emojis
    onlineStatus: str
    badgeRoles: List[BadgeRoles]
    url: Union[str, None]
    uri: Union[str, None]
    movedTo: Union[str, None]
    alsoKnownAs: List[Union[str, None]]
    createdAt: str
    updatedAt: Union[str, None]
    lastFetchedAt: Union[str, None]
    bannerUrl: Union[str, None]
    bannerBlurhash: Union[str, None]
    isLocked: bool
    isSilenced: bool
    isLimited: bool
    isSuspended: bool
    description: Union[str, None]
    location: Union[str, None]
    birthday: Union[str, None]
    lang: Union[str, None]
    fields: List[field]
    verifiedLinks: List[str]
    followersCount: int
    followingCount: int
    notesCount: int
    pinnedNoteIds: List[str]
    pinnedNotes: List[Note]
    pinnedPageId: Union[str, None]
    pinnedPage: Page
    publicReactions: bool
    followingVisibility: str
    followersVisibility: str
    memo: Union[str, None]
    moderationNote: str
    isFollowing: bool
    isFollowed: bool
    hasPendingFollowRequestFromYou: bool
    hasPendingFollowRequestToYou: bool
    isBlocking: bool
    isBlocked: bool
    isMuted: bool
    isRenoteMuted: bool
    notify: str
    withReplies: bool
    avatarId: Union[str, None]
    bannerId: Union[str, None]
    isModerator: Union[bool, None]
    isAdmin: Union[bool, None]
    injectFeaturedNote: bool
    receiveAnnouncementEmail: bool
    alwaysMarkNsfw: bool
    autoSensitive: bool
    carefulBot: bool
    autoAcceptFollowed: bool
    noCrawle: bool
    preventAiLearning: bool
    isExplorable: bool
    isDeleted: bool
    twoFactorBackupCodesStock: str
    hideOnlineStatus: bool
    hasUnreadSpecifiedNotes: bool
    hasUnreadMentions: bool
    hasUnreadAnnouncement: bool
    unreadAnnouncements: List[Announcement]
    hasUnreadAntenna: bool
    hasUnreadChannel: bool
    hasUnreadNotification: bool
    hasPendingReceivedFollowRequest: bool
    unreadNotificationsCount: int
    mutedWords: List[str]
    mutedInstances: List[Union[str, None]]
    notificationRecieveConfig: Any
    emailNotificationTypes: List[str]
    achievements: List[Achievements]
    loggedInDays: int
    policies: RolePolicies
    role: RoleLite
    email: Union[str, None]
    emailVerified: Union[bool, None]
    securityKeysList: List[SecurityKeysList]
    twoFactorEnabled: bool = False
    usePasswordLessLogin: bool = False
    securityKeys: bool = False


@dataclasses.dataclass()
class UserDetailedNotMe:
    id: str
    name: Union[str, None]
    username: str
    host: Union[str, None]
    avatarUrl: Union[str, None]
    avatarBlurhash: Union[str, None]
    avatarDecorations: AvatarDecorations
    isBot: bool
    isCat: bool
    instance: Instance
    emojis: Emojis
    onlineStatus: str
    badgeRoles: List[BadgeRoles]
    url: Union[str, None]
    uri: Union[str, None]
    movedTo: Union[str, None]
    alsoKnownAs: List[Union[str, None]]
    createdAt: str
    updatedAt: Union[str, None]
    lastFetchedAt: Union[str, None]
    bannerUrl: Union[str, None]
    bannerBlurhash: Union[str, None]
    isLocked: bool
    isSilenced: bool
    isLimited: bool
    isSuspended: bool
    description: Union[str, None]
    location: Union[str, None]
    birthday: Union[str, None]
    lang: Union[str, None]
    fields: List[field]
    verifiedLinks: List[str]
    followersCount: int
    followingCount: int
    notesCount: int
    pinnedNoteIds: List[str]
    pinnedNotes: List[Note]
    pinnedPageId: Union[str, None]
    pinnedPage: Page
    publicReactions: bool
    followingVisibility: str
    followersVisibility: str
    memo: Union[str, None]
    moderationNote: str
    isFollowing: bool
    isFollowed: bool
    hasPendingFollowRequestFromYou: bool
    hasPendingFollowRequestToYou: bool
    isBlocking: bool
    isBlocked: bool
    isMuted: bool
    isRenoteMuted: bool
    notify: str
    withReplies: bool
    role: RoleLite
    twoFactorEnabled: bool = False
    usePasswordLessLogin: bool = False
    securityKeys: bool = False


@dataclasses.dataclass()
class followers:
    id: str
    createdAt: str
    followeeId: str
    followerId: str
    followee: UserDetailedNotMe
    follower: UserDetailedNotMe


@dataclasses.dataclass()
class reactions:
    id: str
    createdAt: str
    user: UserLite
    type: str


@dataclasses.dataclass()
class Frequently_replied:
    user: Union[UserDetailedNotMe, MeDetailed]
    weight: int


@dataclasses.dataclass()
class Follow:
    id: str
    name: Union[str, None]
    username: str
    host: Union[str, None]
    avatarUrl: Union[str, None]
    avatarBlurhash: Union[str, None]
    avatarDecorations: List[AvatarDecorations]
    isBot: bool
    isCat: bool
    instance: Instance
    emojis: Emojis
    onlineStatus: str
    badgeRoles: BadgeRoles

"""
@dataclasses.dataclass()
class Pin:
    id: str
    createdAt: str
    isLocked: bool
    isSilenced: bool
    isLimited: bool
    isSuspended: bool
    followersCount: int
    followingCount: int
    notesCount: int
    pinnedPage: Page
    publicReactions: bool
    followingVisibility: str
    followersVisibility: str
    moderationNote: str
    isFollowing: bool
    isFollowed: bool
    hasPendingFollowRequestFromYou: bool
    hasPendingFollowRequestToYou: bool
    isBlocking: bool
    isBlocked: bool
    isMuted: bool
    isRenoteMuted: bool
    notify: str
    withReplies: bool
    injectFeaturedNote: bool
    receiveAnnouncementEmail: bool
    alwaysMarkNsfw: bool
    autoSensitive: bool
    carefulBot: bool
    autoAcceptFollowed: bool
    noCrawle: bool
    preventAiLearning: bool
    isExplorable: bool
    isDeleted: bool
    twoFactorBackupCodesStock: str
    hideOnlineStatus: bool
    hasUnreadSpecifiedNotes: bool
    hasUnreadMentions: bool
    hasUnreadAnnouncement: bool
    unreadAnnouncements: List[Announcement]
    hasUnreadAntenna: bool
    hasUnreadChannel: bool
    hasUnreadNotification: bool
    hasPendingReceivedFollowRequest: bool
    unreadNotificationsCount: int
    notificationRecieveConfig: Any
    loggedInDays: int
    policies: RolePolicies
    role: RoleLite
    name: Union[str, None] = None
    username: Union[str, None] = None
    host: Union[str, None] = None
    avatarUrl: Union[str, None] = None
    avatarBlurhash: Union[str, None] = None
    avatarDecorations: List[AvatarDecorations] = Field(default_factory=list)
    isBot: Union[bool, None] = None
    isCat: Union[bool, None] = None
    instance: Union[Instance, None] = None
    emojis: Emojis = Field(default_factory=list)
    onlineStatus: str = Field(default_factory="unknown")
    badgeRoles: List[BadgeRoles] = Field(default_factory=list)
    url: Union[str, None] = None
    uri: Union[str, None] = None
    movedTo: Union[str, None] = None
    alsoKnownAs: List[Union[str, None]] = None
    updatedAt: Union[str, None] = None
    lastFetchedAt: Union[str, None] = None
    bannerUrl: Union[str, None] = None
    bannerBlurhash: Union[str, None] = None
    description: Union[str, None] = None
    location: Union[str, None] = None
    birthday: Union[str, None] = None
    lang: Union[str, None] = None
    Fields: List[Field] = Field(default_factory=list)
    verifiedLinks: List[str] = Field(default_factory=list)
    pinnedNoteIds: List[str] = Field(default_factory=list)
    pinnedNotes: List[Note] = Field(default_factory=list)
    pinnedPageId: Union[str, None] = None
    memo: Union[str, None] = None

    avatarId: Union[str, None] = None
    bannerId: Union[str, None] = None
    isModerator: Union[bool, None] = None
    isAdmin: Union[bool, None] = None

    mutedWords: List[str] = Field(default_factory=list)
    mutedInstances: List[Union[str, None]] = None
    emailNotificationTypes: List[str] = Field(default_factory=list)
    achievements: List[Achievements] = Field(default_factory=list)

    email: Union[str, None] = None
    emailVerified: Union[bool, None] = None
    securityKeysList: List[SecurityKeysList] = Field(default_factory=list)
    twoFactorEnabled: bool = False
    usePasswordLessLogin: bool = False
    securityKeys: bool = False
"""


from __future__ import annotations

from typing import List, Union

from mitypes.drive import DriveFile
from mitypes.user import UserLite
from pydantic import dataclasses


@dataclasses.dataclass()
class PageBlock:
    pass


@dataclasses.dataclass()
class Properties:
    width: int
    height: int
    orientation: int
    avgColor: str


@dataclasses.dataclass()
class Folder:
    id: str
    createdAt: str
    name: str
    parentId: Union[str, None]
    foldersCount: int
    filesCount: int
    parent: Folder


@dataclasses.dataclass()
class eyeCatchingImage:
    id: str
    createdAt: str
    name: str
    type: str
    md5: str
    size: int
    isSensitive: bool
    blurhash: Union[str, None]
    properties: Properties
    url: str
    thumbnailUrl: Union[str, None]
    comment: Union[str, None]
    folderId: Union[str, None]
    folder: Folder


@dataclasses.dataclass()
class Page:
    id: str
    createdAt: str
    updatedAt: str
    userId: str
    user: UserLite
    content: PageBlock
    variables: List[object]
    title: str
    name: str
    summary: Union[str, None]
    hideTitleWhenPinned: bool
    alignCenter: bool
    font: str
    script: str
    eyeCatchingImageId: Union[str, None]
    eyeCatchingImage: eyeCatchingImage
    attachedFiles: List[DriveFile]
    likedCount: int
    isLiked: bool


@dataclasses.dataclass()
class likes:
    id: str
    page: Page



from typing import Any

from pydantic import dataclasses


@dataclasses.dataclass
class reactions:
    create: Any
    delete: Any


@dataclasses.dataclass
class APIAction:
    reply: Any
    renote: Any
    reactions: reactions



from typing import Union

from mitypes import UserLite
from pydantic import dataclasses


@dataclasses.dataclass()
class clips:
    id: str
    createdAt: str
    lastClippedAt: Union[str, None]
    userId: str
    user: UserLite
    name: str
    description: Union[str, None]
    isPublic: bool
    favoritedCount: int
    isFavorited: bool



import asyncio
import inspect
import logging
import traceback
from enum import Enum
from functools import partial, reduce

from importlib import import_module
from typing import Union

from mitypes.user import User

from .core.exception import (
    ClientException,
    MisskeyAPIError,
)

#     NotExtensionError
from .core.http import AsyncHttpHandler, HttpHandler
from .core.models.internal import error
from .core.models.note import Context
from .core.websocket import MiWS_V2
from .core.experimental.aiows import MSC

# from .core.models.note import Context
from .endpoints.drive import drive
from .endpoints.i import antennas, blocking, following, mute, users
from .endpoints.i import i as ep_i
from .endpoints.notes import notes
from .endpoints.reaction import reactions


class Timeline(Enum):
    HOME = "homeTimeline"
    LOCAL = "localTimeline"
    SOCIAL = "hybridTimeline"
    HYBRID = "hybridTimeline"
    GLOBAL = "globalTimeline"


class Bot:
    def __init__(self, address: str, i: Union[str, None], use_ws_aio: bool=False) -> None:
        self.apierrors = []
        self.logger = logging.getLogger("misspy")
        self.address = address
        self._i = i
        self.engine = MiWS_V2 if not use_ws_aio else MSC
        self.ssl = True
        self.http = AsyncHttpHandler(
            self.address, self._i, self.ssl, logger=self.logger
        )
        self.http_sync = HttpHandler(self.address, self._i, self.ssl)
        self.user: User = User(**self.__i())
        self.__funcs: dict = {}
        self.cls = []
        self.endpoint_list = self.endpoints()
        args = {
            "address": self.address,
            "i": self._i,
            "ssl": self.ssl,
            "endpoints": self.endpoint_list,
            "handler": self.http,
        }
        # ---------- endpoints ------------
        self.antennas = antennas(**args)
        self.notes = notes(**args)
        self.drive = drive(**args)
        self.reactions = reactions(**args)
        self.i = ep_i(**args)
        self.blocking = blocking(**args)
        self.mute = mute(**args)
        self.users = users(**args)
        self.following = following(**args)
        # ---------------------------------
        self.reconnectionCoolDown: int = 3

        self.__connected_channels: list = []

    def __i(self):
        return self.http_sync.send("i", data={})

    def endpoints(self):
        return self.http_sync.send("endpoints", data={})

    def run(self, reconnect=False, log_handler: logging.Logger="INTERNAL"):
        asyncio.run(self.start())

    async def __reconnecter(self):
        for channel in self.__connected_channels:
            await self.connect(channel["channel"], channel["id"])
        self.__funcs["on_ready"].pop(0)

    async def start(self, reconnect=False):
        self.ws = self.engine(self.address, self._i, self.handler, reconnect, self.ssl)
        await self.handler({"type": "__internal", "body": {"type": "setup_hook"}})
        try:
            await self.ws.start()
        except Exception as e:
            if isinstance(e, self.ws.ConnectionClosedError):
                if reconnect:
                    if isinstance(self.reconnectionCoolDown, int):
                        await asyncio.sleep(self.reconnectionCoolDown)
                    else:
                        await asyncio.sleep(3)
                    if self.__funcs.get("on_ready"):
                        self.__funcs["on_ready"].insert(0, self.__reconnecter)
                    else:
                        self.__funcs["on_ready"] = [self.__reconnecter]
                    await self.start(reconnect)

                else:
                    await self.handler(
                        {
                            "type": "__internal",
                            "body": {
                                "type": "exception",
                                "errorType": e.__class__.__name__,
                                "exc": traceback.format_exc(),
                                "exc_obj": e,
                            },
                        }
                    )
            await self.handler(
                {
                    "type": "__internal",
                    "body": {
                        "type": "exception",
                        "errorType": e.__class__.__name__,
                        "exc": traceback.format_exc(),
                        "exc_obj": e,
                    },
                }
            )

    async def include_gear(self, package):
        module = import_module(package).gear
        self.__funcs = reduce(lambda d1, d2: {key: d1.get(key, []) + d2.get(key, []) for key in set(d1) | set(d2)}, [module._funcs, self.__funcs])

    def event(self, event=""):
        """A decorator that can listen for events in Discord.py-like notation.

        For a list of events, see [documentation](https://docs.misspy.xyz/rewrite/events).

        ## Examples:
        ```python
        @bot.event()
        async def on_ready():
            print("ready!")
        ```

        ↓ Put the event name in the decorator argument.
        ```python
        @bot.event("ready")
        async def ready():
            print("ready!")
        ```

        Args:
            event (str): Name of the event to listen for.
        """

        def decorator(func):
            if event == "":
                ev = func.__name__
            else:
                ev = event
            func.__event_type = ev
            if isinstance(func, staticmethod):
                func = func.__func__
            if not inspect.iscoroutinefunction(func):
                raise TypeError("Functions that listen for events must be coroutines.")
            if self.__funcs.get(ev) and isinstance(self.__funcs.get(ev), list):
                ev: list = self.__funcs.get(ev)
                ev.append(func)
            else:
                self.__funcs[ev] = [func]
            return func

        return decorator

    """
    async def load_extension(self, module: str):
        module = import_module(module)
        try:
            await module.setup(self)
        except AttributeError:
            raise NotExtensionError(
                "Module loading failed because the setup function does not exist in the module."
            )
    """

    async def connect(self, channel: Union[str, Timeline], id=None):
        if isinstance(channel, Timeline):
            channel = channel.value
        await self.ws.connect_channel(channel, id)
        self.__connected_channels.append({"channel": channel, "id": id})

    async def handler(self, json: dict):
        try:
            if json["type"] == "channel":
                if json["body"]["type"] == "note":
                    if self.__funcs.get("on_note") is None:
                        return
                    json["body"]["body"]["api"] = {}
                    json["body"]["body"]["api"]["reactions"] = {}
                    json["body"]["body"]["api"]["reactions"]["create"] = partial(
                        self.reactions.create, noteId=json["body"]["body"]["id"]
                    )
                    json["body"]["body"]["api"]["reactions"]["delete"] = partial(
                        self.reactions.delete, noteId=json["body"]["body"]["id"]
                    )
                    json["body"]["body"]["api"]["reply"] = partial(
                        self.notes.create, replyId=json["body"]["body"]["id"]
                    )
                    json["body"]["body"]["api"]["renote"] = partial(
                        self.notes.create, renoteId=json["body"]["body"]["id"]
                    )
                    pnote = Context(**json["body"]["body"])

                    for func in self.__funcs["on_note"]:
                        await func(pnote)
                if json["body"]["type"] == "followed":
                    if self.__funcs.get("on_followed") is None:
                        return
                    for func in self.__funcs["on_followed"]:
                        await func()
        except Exception as e:
            await self.handler(
                {
                    "type": "__internal",
                    "body": {
                        "type": "exception",
                        "errorType": e.__class__.__name__,
                        "exc": traceback.format_exc(),
                        "exc_obj": e,
                    },
                }
            )
        if json["type"] == "__internal":
            if json["body"]["type"] == "setup_hook":
                if self.__funcs.get("setup_hook") is None:
                    return
                for func in self.__funcs["setup_hook"]:
                    await func()
            if json["body"]["type"] == "ready":
                if self.__funcs.get("on_ready") is None:
                    return
                for func in self.__funcs["on_ready"]:
                    await func()
            elif json["body"]["type"] == "exception":
                if self.__funcs.get("on_error"):
                    eb = {
                        "type": json["body"]["errorType"],
                        "exc": json["body"]["exc"],
                        "exc_obj": json["body"]["exc_obj"],
                    }
                    for func in self.__funcs["on_error"]:
                        await func(error(**eb))
                else:
                    if json["body"]["errorType"] in self.apierrors:
                        raise MisskeyAPIError(json["body"]["exc"])
                    else:
                        raise ClientException(json["body"]["exc"])



import inspect


class Gear:
    def __init__(self) -> None:
        self._funcs: dict = {}
    
    def listen(self, event=""):
        """A decorator that can listen for events in Discord.py-like notation.

        For a list of events, see [documentation](https://misspy.github.io/docs/events).

        ## Examples:
        ```python
        gear = misspy.Gear()
        
        @gear.listen()
        async def on_ready():
            print("ready!")
        ```

        ↓ Put the event name in the decorator argument.
        ```python
        gear = misspy.Gear()
        
        @gear.listen("ready")
        async def ready():
            print("ready!")
        ```

        Args:
            event (str): Name of the event to listen for.
        """

        def decorator(func):
            if not hasattr(self, "_funcs"):
                self._funcs = {}
            if event == "":
                ev = func.__name__ 
            else:
                ev = event
            func.__event_type = ev
            if isinstance(func, staticmethod):
                func = func.__func__
            if not inspect.iscoroutinefunction(func):
                raise TypeError("Functions that listen for events must be coroutines.")
            if self._funcs.get(ev) and isinstance(self._funcs.get(ev), list):
                ev: list = self._funcs.get(ev)
                ev.append(func)
            else:
                self._funcs[ev] = [func]
            return func
        return decorator


from typing import List, Union

from pydantic import dataclasses

@dataclasses.dataclass()
class App:
    id: str
    name: str
    callbackUrl: str
    permission: List[str]
    secret: str


@dataclasses.dataclass()
class SecurityKeysList:
    id: str
    name: str
    lastUsed: str


@dataclasses.dataclass()
class RolePolicies:
    gtlAvailable: bool
    ltlAvailable: bool
    canPublicNote: bool
    canInitiateConversation: bool
    canCreateContent: bool
    canUpdateContent: bool
    canDeleteContent: bool
    canPurgeAccount: bool
    canUpdateAvatar: bool
    canUpdateBanner: bool
    mentionLimit: int
    canInvite: bool
    inviteLimit: int
    inviteLimitCycle: int
    inviteExpirationTime: int
    canManageCustomEmojis: bool
    canManageAvatarDecorations: bool
    canSearchNotes: bool
    canUseTranslator: bool
    canUseDriveFileInSoundSettings: bool
    canHideAds: bool
    driveCapacityMb: int
    alwaysMarkNsfw: bool
    skipNsfwDetection: bool
    pinLimit: int
    antennaLimit: int
    antennaNotesLimit: int
    wordMuteLimit: int
    webhookLimit: int
    clipLimit: int
    noteEachClipsLimit: int
    userListLimit: int
    userEachUserListsLimit: int
    rateLimitFactor: int
    avatarDecorationLimit: int


@dataclasses.dataclass()
class Achievements:
    name: str
    unlockedAt: int


@dataclasses.dataclass()
class Announcement:
    id: str
    createdAt: str
    updatedAt: Union[str, None]
    text: str
    title: str
    imageUrl: Union[str, None]
    icon: str
    display: str
    needConfirmationToRead: bool
    forYou: bool
    closeDuration: int
    displayOrder: int
    silence: bool
    isRead: bool



from typing import List, Union

import aiohttp
from mitypes.user import User

from ...core.exception import AuthFailed
from ...core.models.internal import auth_session
from ...core.models.other import App


class session:
    def __init__(self) -> None:
        pass

    async def generate(
        self,
        host: str,
        name: str,
        description: str,
        permission: List[str],
        callback: Union[str, None] = None,
    ):
        async with aiohttp.ClientSession() as session:
            async with session.post(
                "https://{}/api/app/create".format(host),
                data={
                    "name": name,
                    "description": description,
                    "permission": permission,
                    "callbackUrl": callback,
                },
            ) as resp:
                authApp = App(**await resp.json())
                async with session.post(
                    "https://{}/api/auth/session/generate".format(host),
                    data={"appSecret": authApp.secret},
                ) as resp2:
                    resp2 = await resp2.json()
                    return auth_session(
                        host, authApp.secret, resp2["token"], resp2["url"]
                    )

    async def check(self, authSession: auth_session):
        async with aiohttp.ClientSession() as session:
            async with session.post(
                "https://{}/api/auth/session/userkey".format(authSession.host)
            ) as resp:
                resp = await resp.json()
                if resp.get("accessToken") is not None:
                    resp_i: dict = resp["user"]
                    resp_i["accessToken"] = resp["accessToken"]
                    return User(**resp_i)
                else:
                    raise AuthFailed(resp)



from typing import Union

from pydantic import dataclasses

@dataclasses.dataclass()
class RoleLite:
    id: str
    name: str
    color: Union[str, None]
    iconUrl: Union[str, None]
    description: str
    isModerator: bool
    isAdminstrator: bool
    displayOrder: int



from typing import Union

from mitypes.drive import DriveFile
from mitypes.user import UserLite
from pydantic import dataclasses


@dataclasses.dataclass()
class GalleryPost:
    id: str
    createdAt: str
    updatedAt: str
    userId: str
    user: UserLite
    title: str
    description: Union[str, None]
    fileIds: list[str]
    files: list[DriveFile]
    tags: list[str]
    isSensitive: bool
    likedCount: int
    isLikes: bool


@dataclasses.dataclass()
class Like:
    id: str
    post: GalleryPost



import traceback

import websockets
import orjson

class MiWS_V2:
    """
    Version 2 of "MiWS". Speeds may be slower than MSC, but relatively stable.
    """

    def __init__(
        self,
        address: str,
        i: str,
        handler,
        reconnect: bool=False,
        ssl: bool=True
    ) -> None:
        """Version 2 of "MiWS". Speeds may be slower than MSC, but relatively stable.

        Args:
            address (str): Misskey Server Address
            i (str): Misskey API Token
            handler (function): Function to receive json received via Websocket
            reconnect (bool, optional): If True, reconnects on disconnect. Defaults to False.
        """
        self.address = address
        self.i = i
        self.reconnect = reconnect
        self.handler = handler
        self.urlfmt = "://"
        if ssl:
            self.urlfmt = "s://"
        self.ConnectionClosedError = websockets.ConnectionClosedError

    async def close(self):
        await self.ws.close()

    async def start(self):
        async with websockets.connect('ws{}{}/streaming?i={}'.format(self.urlfmt, self.address, self.i)) as self.ws:
            await self.handler({"type": "__internal", "body": {"type": "ready"}})
            recv = orjson.loads(await self.ws.recv())
            await self.handler(recv)
            while True:
                try:
                    recv = orjson.loads(await self.ws.recv())
                    await self.handler(recv)
                except Exception as e:
                    await self.handler({"type": "__internal", "body": {"type": "exception", "errorType": e.__class__.__name__, "exc": traceback.format_exc(), "exc_obj": e}})
                """
                try:
                    recv = orjson.loads(await self.ws.recv())
                    await self.handler(recv)
                except:
                    if self.reconnect:
                        raise TryAgain
                    else:
                        raise websockets.ConnectionClosed
                """

    async def connect_channel(self, channel: str, id: str=None): 
        if id is None:
            id = channel
        await self.ws.send(orjson.dumps({"type": 'connect', "body": {"channel": channel, "id": id, "params": {}}}).decode('utf-8'))


"""
aiohttp Engine
"""
import traceback

import aiohttp

class MSC:
    """
    MSC (Misskey Streaming Client) is an asynchronous client for Misskey StreamingAPI that rewrites MiWS in aiohttp.
    """

    def __init__(
        self,
        address: str,
        i: str,
        handler,
        reconnect: bool=False,
        ssl: bool=True
    ) -> None:
        """MSC (Misskey Streaming Client) is an asynchronous client for Misskey StreamingAPI that rewrites MiWS in aiohttp.

        Args:
            address (str): Misskey Server Address
            i (str): Misskey API Token
            handler (function): Function to receive json received via Websocket
            reconnect (bool, optional): If True, reconnects on disconnect. Defaults to False.
        """
        self.address = address
        self.i = i
        self.reconnect = reconnect
        self.handler = handler
        self.session = aiohttp.ClientSession()
        self.urlfmt = "://"
        if ssl:
            self.urlfmt = "s://"

    async def start(self):
        async with self.session.ws_connect('ws{}{}/streaming?i={}'.format(self.urlfmt, self.address, self.i)) as self.ws:
            await self.handler({"type": "__internal", "body": {"type": "ready"}})
            recv = await self.ws.receive_json()
            await self.handler(recv)
            while True:
                try:
                    recv = await self.ws.receive_json()
                    await self.handler(recv)
                except Exception as e:
                    await self.handler({"type": "__internal", "body": {"type": "exception", "errorType": e.__class__.__name__, "exc": traceback.format_exc(), "exc_obj": e}})

    async def connect_channel(self, channel: str, id: str=None): 
        if id is None:
            id = channel
        await self.ws.send_json({"type": 'connect', "body": {"channel": channel, "id": id, "params": {}}})


from importlib.metadata import version

from mitypes.drive import DriveFile, DriveFolder, df_property
from mitypes.poll import Poll
from mitypes.user import AvatarDecorations, User, UserLite

from .Bot import Bot, Timeline
from .gear import Gear
from .core import websocket
from .core.experimental import aiows
from .core.models import *  # noqa: F403
from .core.models.note import Context, Note

__version__ = version("misspy")

MSC = aiows.MSC  # Misskey
MIWS_V2 = websocket.MiWS_V2



from typing import Union

from .users import UserDetailedNotMe
from pydantic import dataclasses


@dataclasses.dataclass()
class mute:
    id: str
    createdAt: str
    expiresAt: Union[str, None]
    muteeId: str
    mutee: UserDetailedNotMe



import inspect


class Cog:
    @classmethod
    def listener(cls, event=""):
        """A decorator that can listen for events in Discord.py-like notation.

        Examples:
        ```python
        @misspy.Cog.listen("ready")
        async def ready():
            print("ready!")
        ```

        Args:
            event (str): Name of the event to listen for.
        """
        if event == "":
            raise TypeError("Error in Cog.listener: event name is Missing.")

        def decorator(func):
            if isinstance(func, staticmethod):
                func = func.__func__
            if not inspect.iscoroutinefunction(func):
                raise TypeError("Functions that listen for events must be coroutines.")
            func._event = event
            return classmethod(func)

        return decorator





def to(address: str, protocol: str, ssl=True):
    http = protocol + "://"
    https = protocol + "s://"
    if not address.startswith(http) and not address.startswith(https):
        address = http + address
        if ssl:
            address = https + address
        address_raw = address
    else:
        address = address
        address_raw = address.replace(https, "").replace(http, "")
    return {"address": address, "raw": address_raw}


from typing import List, Union

from mitypes import channel

from ..core.http import AsyncHttpHandler
from ..utils.internaltool import nonecheck


class channels:
    def __init__(
        self,
        address: str,
        i: Union[str, None],
        ssl: bool,
        endpoints: List[str],
        handler: AsyncHttpHandler = None,
    ) -> None:
        self.http = handler
        if handler is None:
            self.http = AsyncHttpHandler(address, i, ssl)
        self.endpoints = endpoints

    async def create(self, name, description=None, bannerId=None):
        return channel(
            **await self.http.send(
                "channels/create",
                {"name": name, "description": description, "bannerId": bannerId},
            )
        )

    async def update(self, channelId, name, description=None, bannerId=None):
        return channel(
            **await self.http.send(
                "channels/update",
                {
                    "channelId": channelId,
                    "name": name,
                    "description": description,
                    "bannerId": bannerId,
                },
            )
        )

    async def follow(self, channelId):
        return await self.http.send("channels/follow", {"channelId": channelId})

    async def unfollow(self, channelId):
        return await self.http.send("channels/unfollow", {"channelId": channelId})

    async def show(self, channelId):
        return channel(
            **await self.http.send("channels/show", {"channelId": channelId})
        )

    async def followed(self, sinceId=None, untilId=None, limit=5):
        base = {"limit": limit}
        if nonecheck(sinceId):
            base["sinceId"] = sinceId
        if nonecheck(untilId):
            base["untilId"] = untilId
        data = []
        chs = await self.http.send("channels/followed", base)
        for ch in chs:
            data.append(channel(**ch))
        return data

    async def owned(self, limit=5, sinceId=None, untilId=None):
        base = {"limit": limit}
        if nonecheck(sinceId):
            base["sinceId"] = sinceId
        if nonecheck(untilId):
            base["untilId"] = untilId
        data = []
        chs = await self.http.send("channels/owned", base)
        for ch in chs:
            data.append(channel(**ch))
        return data

    async def featured(self):
        data = []
        chs = await self.http.send("channels/featured", {})
        for ch in chs:
            data.append(channel(**ch))
        return data



from enum import Enum
from typing import Union

from mitypes.user import UserLite
from pydantic import dataclasses

from .note import Note

class AnnounceIconEnum(str, Enum):
    INFO = "info"
    WARNING = "warning"
    ERROR = "error"
    SUCCESS = "success"

class AnnounceDisplayEnum(str, Enum):
    DIALOG = "dialog"
    NORMAL = "normal"
    BANNER = "banner"

@dataclasses.dataclass()
class Announce:
    id: str
    createdAt: str
    updatedAt: Union[str, None]
    text: str
    title: str
    imageUrl: Union[str, None]
    icon: AnnounceIconEnum
    display: AnnounceDisplayEnum
    needConfirmationToRead: bool
    silence: bool
    forYou: bool
    isRead: bool

@dataclasses.dataclass()
class notifications:
    id: str
    createdAt: str
    type: str
    user: UserLite
    userId: str
    note: Note


import traceback
from enum import Enum
from typing import List, Union

import orjson
import pydantic
from mitypes.drive import DriveFile
from mitypes.poll import Poll

from ..core.exception import NotFound
from ..core.http import AsyncHttpHandler
from ..core.models.note import Note
from ..utils.internaltool import nonecheck


class Visibility(Enum):
    PUBLIC = "public"
    HOME = "home"
    FOLLOWERS = "followers"
    SPEFICED = "specified"


class notes:
    def __init__(
        self,
        address: str,
        i: Union[str, None],
        ssl: bool,
        endpoints: List[str],
        handler: AsyncHttpHandler = None,
    ) -> None:
        self.__http = handler
        if handler is None:
            self.__http = AsyncHttpHandler(address, i, ssl)
        self.endpoints = endpoints

    async def create(
        self,
        text: Union[str, None] = None,
        visibility: Union[str, Visibility] = "public",
        visibleUserIds: List[str] = [],
        cw: Union[str, None] = None,
        localOnly: bool = False,
        noExtractMentions: bool = False,
        noExtractHashtags: bool = False,
        noExtractEmojis: bool = False,
        fileIds: Union[List[DriveFile], None] = None,
        replyId: Union[str, None] = None,
        renoteId: Union[str, None] = None,
        channelId: Union[str, None] = None,
        poll: Union[Poll, None] = None,
    ) -> Note:
        """Create a post. Replies and Renotes are also made using this function.

        Args:
            text (Union[str, None], optional): The text of the post. Defaults to None.
            visibility (str, optional): Publication range of posts. Defaults to visibility.public.
            visibleUserIds (List[str], optional): A list of user IDs that can view the post. Applies only when visibility is 'specified'.
            cw (Union[str, None], optional): CW (Content Warning) of the post. Defaults to None.
            localOnly (bool, optional): If set to True, posts will only be posted locally. Defaults to False.
            noExtractMentions (bool, optional): If set to True, mentions will not be expanded from the body. Defaults to False.
            noExtractHashtags (bool, optional): If set to True, hashtags will not be expanded from the body. Defaults to False.
            noExtractEmojis (bool, optional): If set to True, emojis will not be expanded from the body. Defaults to False.
            fileIds (List[str], optional): The id of the file to attach to the post.
            replyId (Union[str, None], optional): The id of the post to reply to. Defaults to None.
            renoteId (Union[str, None], optional): ID of the post targeted for Renote. Defaults to None.
            channelId (Union[str, None], optional): ID of the channel to post to. Defaults to None.
            poll (Union[Poll, None], optional): Voting parameters. Defaults to None.

        Returns:
            Note: created Post.
        """
        endpoint = "notes/create"
        if endpoint not in self.endpoints:
            raise NotFound('endpoint "notes/create" is not found.')

        data = {
            "text": text,
            "visibility": visibility,
            "visibleUserIds": visibleUserIds,
            "cw": cw,
            "localOnly": localOnly,
            "noExtractMentions": noExtractMentions,
            "noExtractHashtags": noExtractHashtags,
            "noExtractEmojis": noExtractEmojis,
            "replyId": replyId,
            "renoteId": renoteId,
            "channelId": channelId,
        }
        if isinstance(visibility, Visibility):
            data["visibility"] = visibility.value
        if fileIds:
            fid = []
            for file in fileIds:
                fid.append(file.id)
            data["fileIds"] = fid
        if poll:
            data["poll"] = poll
        resp = await self.__http.send(endpoint, data=data)
        try:
            return Note(**resp["createdNote"])
        except pydantic.ValidationError:
            exc = traceback.format_exc()
            raise ValueError(
                exc + "\n\nError JSON: " + orjson.dumps(resp).decode("utf-8")
            )

    async def delete(self, noteId: str) -> bool:
        """delete post.

        Args:
            noteId (str): Post id.

        Returns:
            bool: Hi
        """
        endpoint = "notes/delete"
        if endpoint not in self.endpoints:
            raise NotFound(f'endpoint "{endpoint}" is not found.')
        data = {"noteId": noteId}
        return await self.__http.send(endpoint, data=data)

    async def conversation(self, noteId: str, limit: int = 10, offset: int = 0):
        """Retrieve relevant notes.

        Args:
            noteId (str): noteId.
            limit (int, optional): limit of Retrieve. Defaults to 10.
            offset (int, optional): Skip the first offset of the result. Defaults to 0.

        Returns:
            _type_: _description_
        """
        endpoint = "notes/conversation"
        if endpoint not in self.endpoints:
            raise NotFound(f'endpoint "{endpoint}" is not found.')
        return [
            Note(**r)
            for r in await self.__http.send(
                endpoint,
                {"noteId": noteId, "limit": limit, "offset": offset},
            )
        ]

    async def global_timeline(
        self,
        withFiles=False,
        limit=10,
        sinceId=None,
        untilId=None,
        sinceDate=None,
        untilDate=None,
    ):
        """Get the Global Timeline (GTL). The global timeline contains all public posts received by the server.

        Args:
            withFiles (bool, optional): If set to true, only notes with files attached will be retrieved.
            limit (int, optional): Specifies the maximum number of notes to be retrieved. Defaults to 10.
            sinceId (str, optional): If specified, returns notes whose id is greater than its value. Defaults to None.
            untilId (str, optional): If specified, returns notes whose id is less than the value. Defaults to None.
            sinceDate (int, optional): If you specify a date and time in epoch seconds, it returns notes created after that date and time.
            untilDate (int, optional): If you specify a date and time in epoch seconds, it returns notes created before that date and time.

        Returns:
            List (in Note): notes dict
        """
        base = {"withFiles": withFiles, "limit": limit}
        if nonecheck(sinceId):
            base["sinceId"] = sinceId
        if nonecheck(untilId):
            base["untilId"] = untilId
        if nonecheck(sinceDate):
            base["sinceDate"] = sinceDate
        if nonecheck(untilDate):
            base["untilDate"] = untilDate
        _notes = await self.__http.send("notes/global-timeline", base)
        return [Note(**r) for r in _notes]

    async def hybrid_timeline(
        self,
        limit=10,
        sinceId=None,
        untilId=None,
        sinceDate=None,
        untilDate=None,
        includeMyRenotes=True,
        includeRenotedMyNotes=True,
        includeLocalRenotes=True,
        withFiles=False,
    ):
        """Get the social Timeline (STL). The social timeline includes all public notes in the server and those of users you follow.

        Args:
            limit (int, optional): Specifies the maximum number of notes to be retrieved. Defaults to 10.
            sinceId (str, optional): If specified, returns notes whose id is greater than its value. Defaults to None.
            untilId (str, optional): If specified, returns notes whose id is less than the value. Defaults to None.
            sinceDate (int, optional): If you specify a date and time in epoch seconds, it returns notes created after that date and time.
            untilDate (int, optional): If you specify a date and time in epoch seconds, it returns notes created before that date and time.
            includeMyRenotes (bool, optional): If true, include the renotes made by the currently logged in user.
            includeRenotedMyNotes (bool, optional): If true, include the Renotes posted by the currently logged in user.
            includeLocalRenotes (bool, optional): If true, include the renotes made by local users.
            withFiles (bool, optional): If set to true, only notes with files attached will be retrieved.

        Returns:
            List (in Dict): notes dict
        """
        base = {
            "withFiles": withFiles,
            "limit": limit,
            "includeMyRenotes": includeMyRenotes,
            "includeRenotedMyNotes": includeRenotedMyNotes,
            "includeLocalRenotes": includeLocalRenotes,
        }
        if nonecheck(sinceId):
            base["sinceId"] = sinceId
        if nonecheck(untilId):
            base["untilId"] = untilId
        if nonecheck(sinceDate):
            base["sinceDate"] = sinceDate
        if nonecheck(untilDate):
            base["untilDate"] = untilDate
        _notes = await self.__http.send("notes/hybrid-timeline", base)
        return [Note(**r) for r in _notes]

    async def home_timeline(
        self,
        limit=10,
        sinceId=None,
        untilId=None,
        sinceDate=None,
        untilDate=None,
        includeMyRenotes=True,
        includeRenotedMyNotes=True,
        includeLocalRenotes=True,
        withFiles=False,
    ):
        """Get Home Timeline (HTL). The home timeline contains the notes of the users you follow.

        Args:
            limit (int, optional): Specifies the maximum number of notes to be retrieved. Defaults to 10.
            sinceId (str, optional): If specified, returns notes whose id is greater than its value. Defaults to None.
            untilId (str, optional): If specified, returns notes whose id is less than the value. Defaults to None.
            sinceDate (int, optional): If you specify a date and time in epoch seconds, it returns notes created after that date and time.
            untilDate (int, optional): If you specify a date and time in epoch seconds, it returns notes created before that date and time.
            includeMyRenotes (bool, optional): If true, include the renotes made by the currently logged in user.
            includeRenotedMyNotes (bool, optional): If true, include the Renotes posted by the currently logged in user.
            includeLocalRenotes (bool, optional): If true, include the renotes made by local users.
            withFiles (bool, optional): If set to true, only notes with files attached will be retrieved.

        Returns:
            List (in Dict): notes dict
        """
        base = {
            "withFiles": withFiles,
            "limit": limit,
            "includeMyRenotes": includeMyRenotes,
            "includeRenotedMyNotes": includeRenotedMyNotes,
            "includeLocalRenotes": includeLocalRenotes,
        }
        if nonecheck(sinceId):
            base["sinceId"] = sinceId
        if nonecheck(untilId):
            base["untilId"] = untilId
        if nonecheck(sinceDate):
            base["sinceDate"] = sinceDate
        if nonecheck(untilDate):
            base["untilDate"] = untilDate
        _notes = await self.__http.send("notes/timeline", base)
        return [Note(**r) for r in _notes]

    async def local_timeline(
        self,
        withFiles=False,
        fileType=None,
        excludeNsfw=False,
        limit=10,
        sinceId=None,
        untilId=None,
        sinceDate=None,
        untilDate=None,
    ):
        """Get the Local Timeline (LTL). The local timeline contains all public notes in the server.

        Args:
            withFiles (bool, optional): If set to true, only notes with files attached will be retrieved.
            fileType (bool, optional): Retrieve only those posts with files of the specified type attached.
            excludeNsfw (bool, optional): If true, excludes notes with CWs and notes with NSFW-specified files attached, effective only if fileType is specified (notes with CWs without attachments are not excluded).
            limit (int, optional): Specifies the maximum number of notes to be retrieved. Defaults to 10.
            sinceId (str, optional): If specified, returns notes whose id is greater than its value. Defaults to None.
            untilId (str, optional): If specified, returns notes whose id is less than the value. Defaults to None.
            sinceDate (int, optional): If you specify a date and time in epoch seconds, it returns notes created after that date and time.
            untilDate (int, optional): If you specify a date and time in epoch seconds, it returns notes created before that date and time.
            includeMyRenotes (bool, optional): If true, include the renotes made by the currently logged in user.
            includeRenotedMyNotes (bool, optional): If true, include the Renotes posted by the currently logged in user.
            includeLocalRenotes (bool, optional): If true, include the renotes made by local users.

        Returns:
            List (in Dict): notes dict
        """
        base = {"withFiles": withFiles, "limit": limit, "excludeNsfw": excludeNsfw}
        if nonecheck(fileType):
            base["fileType"] = fileType
        if nonecheck(sinceId):
            base["sinceId"] = sinceId
        if nonecheck(untilId):
            base["untilId"] = untilId
        if nonecheck(sinceDate):
            base["sinceDate"] = sinceDate
        if nonecheck(untilDate):
            base["untilDate"] = untilDate
        _notes = await self.__http.send("notes/local-timeline", base)
        return [Note(**r) for r in _notes]

    async def featured(self, limit=10, offset=0):
        """Retrieves highlighted notes. Results are sorted in descending order of note creation time (latest first).

        Args:
            limit (int, optional): Maximum number of notes to be retrieved. Defaults to 10.
            offset (int, optional): The first offset of the search result is skipped. Defaults to 0.

        Returns:
            List (Dict): _description_
        """
        _notes = await self.__http.send(
            "notes/featured", {"limit": limit, "offset": offset}
        )
        return [Note(**r) for r in _notes]

    async def favorites_create(self, noteId):
        """create favorites.

        Args:
            noteId (str): noteId.

        Returns:
            Dict: _description_
        """
        return await self.__http.send("notes/favorites/create", {"noteId": noteId})

    async def favorites_delete(self, noteId):
        """delete favorites

        Args:
            noteId (str): noteId.

        Returns:
            Dict: _description_
        """
        return await self.__http.send("notes/favorites/delete", {"noteId": noteId})

    async def polls_recommendation(self, limit=10, offset=0):
        """Get a list of recommended notes with a survey.

        Args:
            limit (int, optional): Maximum number of notes to be retrieved. Defaults to 10.
            offset (int, optional): The first offset of the search result is skipped. Defaults to 0.

        Returns:
            _type_: _description_
        """
        return await self.__http.send(
            "notes/polls/recommendation", {"limit": limit, "offset": offset}
        )

    async def polls_vote(self, noteId, choice):
        """Vote in the notebook poll. To vote for multiple choices, change the choice and make multiple requests.

        Args:
            noteId (str): ID of the note to which the survey is attached.
            choice (str): Choices to vote on.

        Returns:
            _type_: _description_
        """
        return await self.__http.send(
            "notes/polls/vote", {"noteId": noteId, "choice": choice}
        )

    async def reactions(
        self,
        noteId: str,
        type: str = None,
        limit: int = 10,
        offset: int = 0,
        sinceId: str = None,
        untilId: str = None,
    ):
        base = {"noteId": noteId, "type": type, "limit": limit, "offset": offset}
        if nonecheck(sinceId):
            base["sinceId"] = sinceId
        if nonecheck(untilId):
            base["untilId"] = untilId
        return await self.__http.send("notes/reactions", base)

    async def replies(
        self, noteId: str, sinceId: str = None, untilId: str = None, limit: int = 10
    ):
        base = {"noteId": noteId, "limit": limit}
        if nonecheck(sinceId):
            base["sinceId"] = sinceId
        if nonecheck(untilId):
            base["untilId"] = untilId
        return await self.__http.send("notes/replies", base)

    async def search(
        self,
        reply=False,
        renote=False,
        withFiles=False,
        poll=False,
        sinceId=None,
        untilId=None,
        limit=10,
    ):
        base = {
            "reply": reply,
            "renote": renote,
            "withFiles": withFiles,
            "poll": poll,
            "limit": limit,
        }
        if nonecheck(sinceId):
            base["sinceId"] = sinceId
        if nonecheck(untilId):
            base["untilId"] = untilId
        return await self.__http.send("notes/search-by-tag", base)

    async def search_by_tag(
        self,
        query,
        sinceId=None,
        untilId=None,
        limit=10,
        offset=0,
        host=None,
        userId=None,
        channelId=None,
    ):
        base = {
            "query": query,
            "offset": offset,
            "limit": limit,
            "host": host,
            "userId": userId,
            "channelId": channelId,
        }
        if nonecheck(sinceId):
            base["sinceId"] = sinceId
        if nonecheck(untilId):
            base["untilId"] = untilId
        return await self.__http.send("notes/search", base)

    async def state(self, noteId):
        return await self.__http.send("notes/state", {"noteId": noteId})

    async def show(self, noteId):
        return await self.__http.send("notes/show", {"noteId": noteId})

    async def thread_muting_create(self, noteId):
        return await self.__http.send("notes/thread-muting/create", {"noteId": noteId})

    async def thread_muting_delete(self, noteId):
        return await self.__http.send("notes/thread-muting/delete", {"noteId": noteId})

    async def translate(self, noteId, targetLang):
        return await self.__http.send(
            "notes/translate", {"noteId": noteId, "targetLang": targetLang}
        )

    async def unrenote(self, noteId):
        return await self.__http.send("notes/unrenote", {"noteId": noteId})

    async def user_list_timeline(
        self,
        listId,
        limit=10,
        sinceId=None,
        untilId=None,
        sinceDate=0,
        untilDate=0,
        includeMyRenotes=True,
        includeRenotedMyNotes=True,
        includeLocalRenotes=True,
        withFiles=False,
    ):
        base = {
            "listId": listId,
            "limit": limit,
            "sinceDate": sinceDate,
            "untilDate": untilDate,
            "includeMyRenotes": includeMyRenotes,
            "includeRenotedMyNotes": includeRenotedMyNotes,
            "includeLocalRenotes": includeLocalRenotes,
            "withFiles": withFiles,
        }
        if nonecheck(sinceId):
            base["sinceId"] = sinceId
        if nonecheck(untilId):
            base["untilId"] = untilId
        n = []
        _notes = await self.__http.send("notes/user-list-timeline", base)
        for note in _notes:
            n.append(Note(**note))
        return n

    async def watching_create(self, noteId):
        return await self.__http.send("notes/watching/create", {"noteId": noteId})

    async def watching_delete(self, noteId):
        return await self.__http.send("notes/watching/delete", {"noteId": noteId})

    async def mentions(
        self, following=False, limit=10, sinceId=None, untilId=None, visibility=None
    ):
        base = {"following": following, "limit": limit}
        if nonecheck(sinceId):
            base["sinceId"] = sinceId
        if nonecheck(untilId):
            base["untilId"] = untilId
        if nonecheck(visibility):
            base["visibility"] = visibility
        return await self.__http.send("notes/mentions", base)



class MiError(Exception):
    pass

class HTTPError(Exception):
    pass

class MisskeyAPIError(MiError):
    pass

class ClientException(MiError):
    pass

class ParseError(Exception):
    pass

class NotExtensionError(Exception):
    pass

class RegistedError(Exception):
    pass

class AlreadyRegisted(RegistedError):
    pass

class NotFound(HTTPError):
    pass

class MiAuthFailed(NotFound):
    pass

class AuthFailed(MisskeyAPIError):
    pass


from typing import List, Union

from mitypes import DriveFile

from ..core.exception import NotFound
from ..core.http import AsyncHttpHandler


class drive:
    def __init__(
        self,
        address: str,
        i: Union[str, None],
        ssl: bool,
        endpoints: List[str],
        handler: AsyncHttpHandler = None,
    ) -> None:
        self.http = handler
        if handler is None:
            self.http = AsyncHttpHandler(address, i, ssl)
        self.endpoints = endpoints

    async def files(
        self,
        limit: int = 10,
        sinceId: str = None,
        untilId: str = None,
        folderId: Union[str, None] = None,
        type: Union[str, None] = None,
    ) -> List[DriveFile]:
        """Obtains a list of files in the specified folder or root hierarchy under the logged-in user's drive.

        Args:
            limit (int, optional): Maximum number of files to retrieve. Defaults to 10.
            sinceId (str, optional): If specified, returns files whose id is greater than that value. Defaults to None.
            untilId (str, optional): If specified, returns files whose id is less than that value. Defaults to None.
            folderId (Union[str, None], optional): If the id of the parent folder is null (default), files in the root hierarchy are retrieved. Defaults to None.
            type (Union[str, None], optional): The MIME type of the file. Defaults to None.

        Returns:
            List[DriveFile]: Files
        """
        endpoint = "drive/files"
        if endpoint not in self.endpoints:
            raise NotFound('endpoint "drive/files" is not found.')
        data = {"limit": limit, "folderId": folderId, "type": type}
        if sinceId:
            data["sinceId"] = sinceId
        if untilId:
            data["untilId"] = untilId
        resp = await self.http.send(endpoint, data=data)
        dfiles = []
        for res in resp:
            dfiles.append(DriveFile(**res))
        return dfiles

    async def files_create(self, path: str) -> DriveFile:
        """Uploads files to the drive of the currently logged-in user.

        Args:
            path (str): file path.

        Returns:
            DriveFile: created file.
        """
        endpoint = "drive/files/create"
        if endpoint not in self.endpoints:
            raise NotFound('endpoint "drive/files/create" is not found.')
        with open(path, "rb") as f:
            return DriveFile(**await self.http.send(endpoint, data={"file": f}))

    async def files_delete(self, fileId: str) -> bool:
        """delete files.

        Args:
            fileId (str): drive file id.

        Returns:
            bool: Hi
        """
        endpoint = "drive/files/delete"
        if endpoint not in self.endpoints:
            raise NotFound('endpoint "drive/files/delete" is not found.')
        return await self.http.send(endpoint, data={"fileId": fileId})



from typing import List, Union, Any

from pydantic import dataclasses, BaseModel, ConfigDict

@dataclasses.dataclass(config=dict(extra="allow"))
class mspy:
    tlId: str


@dataclasses.dataclass(config=dict(extra="allow", arbitrary_types_allowed=True))
class error:
    type: str
    exc: str
    exc_obj: Any


@dataclasses.dataclass()
class miauth_session:
    sessionId: str
    host: str
    url: str
    name: Union[str, None] = None
    icon: Union[str, None] = None
    callback: Union[str, None] = None
    permission: Union[List[str], None] = None


@dataclasses.dataclass()
class auth_session:
    host: str
    appSecret: str
    token: str
    url: str



from typing import List, Union

from mitypes import channel

from ..core.http import AsyncHttpHandler
from ..core.models.antennas import created_antnna, show_antnna


class Antennas:
    def __init__(
        self,
        address: str,
        i: Union[str, None],
        ssl: bool,
        endpoints: List[str],
        handler: AsyncHttpHandler = None,
    ) -> None:
        self.http = handler
        if handler is None:
            self.http = AsyncHttpHandler(address, i, ssl)
        self.endpoints = endpoints

    async def create(
        self,
        name: str,
        src: str,
        userListId: Union[str, None],
        keywords: List[List],
        excludeKeywords: List[List],
        users: List[str],
        caseSensitive: bool=False,
        localOnly: bool=False,
        execludeBots: bool=False,
        withReplies: bool=False,
        withFile: bool=False,
    ):
        resp = await self.http.send(
            "antennas/create",
            {
                "name": name,
                "src": src,
                "userListId": userListId,
                "keywords": keywords,
                "excludeKeywords": excludeKeywords,
                "users": users,
                "caseSensitive": caseSensitive,
                "localOnly": localOnly,
                "execludeBots": execludeBots,
                "withReplies": withReplies,
                "withFile": withFile,
            },
        )
        return created_antnna(
            **resp
        )
